import aiohttp
import urllib.parse
import royalnet.commands as rc
from .lazyplay import LazyplayCommand


class LazyfunkwhaleplaylistCommand(LazyplayCommand):
    name: str = "lazyfunkwhaleplaylist"

    aliases = ["lfwp", "lfwplaylist", "lazyfunkwhalep"]

    description: str = "Cerca una playlist su RoyalWhale e aggiungila (lazy) alla coda della chat vocale."

    syntax = "{ricerca}"

    def get_embed_color(self):
        return 0x009FE3

    async def get_urls(self, args):
        search = urllib.parse.quote(args.joined(require_at_least=1))
        async with aiohttp.ClientSession() as session:
            async with session.get(self.config["Funkwhale"]["instance_url"] +
                                   f"/api/v1/playlists/?q={search}&ordering=-creation_date&playable=true") as response:
                if response.status >= 400:
                    raise rc.ExternalError(f"Request returned {response.status}")
                j = await response.json()
            if len(j["results"]) < 1:
                raise rc.UserError("Nessuna playlist trovata con il nome richiesto.")
            playlist = j["results"][0]
            playlist_id = playlist["id"]
            async with session.get(self.config["Funkwhale"]["instance_url"] +
                                   f"/api/v1/playlists/{playlist_id}/tracks") as response:
                if response.status >= 400:
                    raise rc.ExternalError(f"Request returned {response.status}")
                j = await response.json()
        return list(map(lambda t: f'{self.config["Funkwhale"]["instance_url"]}{t["track"]["listen_url"]}', j["results"]))



# Imports go here!
from .api_bio_get import ApiBioGetStar
from .api_bio_set import ApiBioSetStar
from .api_diario_get import ApiDiarioGetStar
from .api_diario_list import ApiDiarioListStar
from .api_discord_cv import ApiDiscordCvStar
from .api_discord_play import ApiDiscordPlayStar
from .api_wiki_edit import ApiWikiEditStar
from .api_wiki_get import ApiWikiGetStar
from .api_wiki_list import ApiWikiListStar
from .api_fiorygi_get import ApiFiorygiGetStar
from .api_diario_random import ApiDiarioRandomStar
from .api_polls_create import ApiPollsCreate
from .api_polls_get import ApiPollsGet
from .api_polls_list import ApiPollsList
from .api_cvstats_latest import ApiCvstatsLatestStar
from .api_cvstats_avg import ApiCvstatsAvgStar
from .api_user_get_ryg import ApiUserGetRygStar
from .api_user_find_ryg import ApiUserFindRygStar

# Enter the PageStars of your Pack here!
available_page_stars = [
    ApiBioGetStar,
    ApiBioSetStar,
    ApiDiarioGetStar,
    ApiDiarioListStar,
    ApiDiscordCvStar,
    ApiDiscordPlayStar,
    ApiWikiEditStar,
    ApiWikiGetStar,
    ApiWikiListStar,
    ApiFiorygiGetStar,
    ApiDiarioRandomStar,
    ApiPollsCreate,
    ApiPollsGet,
    ApiPollsList,
    ApiCvstatsLatestStar,
    ApiCvstatsAvgStar,
    ApiUserGetRygStar,
    ApiUserFindRygStar,
]

# Don't change this, it should automatically generate __all__
__all__ = [star.__name__ for star in available_page_stars]



import datetime
from typing import *

import discord
import royalnet.commands as rc
import royalnet.serf.discord as rsd
import royalnet.bard.discord as rbd

from ..utils import RoyalQueue, RoyalPool


class DiscordPlaymodeEvent(rc.Event):
    name = "discord_playmode"

    async def run(self,
                  playable_string: str,
                  guild_id: Optional[int] = None,
                  user: Optional[str] = None,
                  **kwargs) -> dict:
        if not isinstance(self.serf, rsd.DiscordSerf):
            raise rc.UnsupportedError()

        serf: rsd.DiscordSerf = self.serf
        client: discord.Client = self.serf.client
        guild: discord.Guild = client.get_guild(guild_id) if guild_id is not None else None
        candidate_players: List[rsd.VoicePlayer] = serf.find_voice_players(guild)

        if len(candidate_players) == 0:
            raise rc.UserError("Il bot non è in nessun canale vocale.\n"
                               "Evocalo prima con [c]summon[/c]!")
        elif len(candidate_players) == 1:
            voice_player = candidate_players[0]
        else:
            raise rc.CommandError("Non so a che Server cambiare Playable...\n"
                                  "Invia il comando su Discord, per favore!")

        if playable_string.upper() == "QUEUE":
            playable = await RoyalQueue.create()
        elif playable_string.upper() == "POOL":
            playable = await RoyalPool.create()
        else:
            raise rc.InvalidInputError(f"Unknown playable '{playable_string.upper()}'")

        await voice_player.change_playing(playable)

        return {
            "name": f"{playable.__class__.__qualname__}"
        }



import royalnet.constellation.api as rca
import royalnet.utils as ru
import royalnet.backpack.tables as rbt
from .api_user_get_ryg import ApiUserGetRygStar


class ApiUserFindRygStar(ApiUserGetRygStar):
    summary = "Ottieni le informazioni su un utente della Royal Games."

    description = ""

    methods = ["GET"]

    path = "/api/user/find/ryg/v1"

    requires_auth = False

    parameters = {"alias": "L'alias dell'utente di cui vuoi vedere le informazioni."}

    tags = ["user"]

    async def get_user(self, data: rca.ApiData):
        user = await rbt.User.find(self.alchemy, data.session, data["alias"])
        if user is None:
            raise rca.NotFoundError("No such user.")
        return user

    async def api(self, data: rca.ApiData) -> dict:
        user = await self.get_user(data)
        result = {
            **user.json(),
            "bio": user.bio.json() if user.bio is not None else None,
            "fiorygi": user.fiorygi.fiorygi if user.fiorygi is not None else None,
            "steam": [steam.json() for steam in user.steam],
            "leagueoflegends": [leagueoflegends.json() for leagueoflegends in user.leagueoflegends],
            "trivia": user.trivia_score.json() if user.trivia_score is not None else None
        }
        return result



from typing import *
from royalnet.constellation.api import *
from royalnet.utils import *
from ..tables import *
from sqlalchemy import func


class ApiDiarioRandomStar(ApiStar):
    path = "/api/diario/random/v1"

    summary = "Get random diario entries."

    parameters = {
        "amount": "The number of diario entries to get."
    }

    tags = ["diario"]

    async def api(self, data: ApiData) -> JSON:
        DiarioT = self.alchemy.get(Diario)
        try:
            amount = int(data["amount"])
        except ValueError:
            raise InvalidParameterError("'amount' is not a valid int.")
        entries: List[Diario] = await asyncify(
            data.session
                .query(DiarioT)
                .order_by(func.random())
                .limit(amount)
                .all
        )
        if len(entries) < amount:
            raise NotFoundError("Not enough diario entries.")
        return list(map(lambda e: e.json(), entries))



from typing import *
import re
import royalnet.commands as rc


class ShipCommand(rc.Command):
    name: str = "ship"

    description: str = "Crea una ship tra due nomi."

    syntax = "{nomeuno} {nomedue}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        name_one = args[0]
        name_two = args[1]
        if name_two == "+":
            name_two = args[2]
        name_one = name_one.lower()
        name_two = name_two.lower()
        # Get all letters until the first vowel, included
        match_one = re.search(r"^[A-Za-z][^aeiouAEIOU]*[aeiouAEIOU]?", name_one)
        if match_one is None:
            part_one = name_one[:int(len(name_one) / 2)]
        else:
            part_one = match_one.group(0)
        # Get all letters from the second to last vowel, excluded
        match_two = re.search(r"[^aeiouAEIOU]*[aeiouAEIOU]?[A-Za-z]$", name_two)
        if match_two is None:
            part_two = name_two[int(len(name_two) / 2):]
        else:
            part_two = match_two.group(0)
        # Combine the two name parts
        mixed = part_one + part_two
        await data.reply(f"💕 {part_one.capitalize()} + {part_two.capitalize()} = [b]{mixed.capitalize()}[/b]")



import enum


class BrawlhallaTier(enum.Enum):
    ZERO = 0
    I = 1
    II = 2
    III = 3
    IV = 4
    V = 5

    def __str__(self):
        return str(self.value)

    def __repr__(self):
        return f"{self.__class__.__qualname__}.{self.name}"

    def __gt__(self, other):
        if other is None:
            return True
        if not isinstance(other, self.__class__):
            raise TypeError(f"Can't compare {self.__class__.__qualname__} with {other.__class__.__qualname__}")
        return self.value > other.value



from typing import *
import royalnet.commands as rc
import aiohttp
import urllib.parse

from .lazyplay import LazyplayCommand


class LazyfunkwhalealbumCommand(LazyplayCommand):
    name: str = "lazyfunkwhalealbum"

    aliases = ["lfwa", "lfwalbum", "lazyfunkwhalea"]

    description: str = "Cerca un album su RoyalWhale e aggiungilo (lazy) alla coda della chat vocale."

    syntax = "{ricerca}"

    def get_embed_color(self):
        return 0x009FE3

    async def get_urls(self, args):
        search = urllib.parse.quote(args.joined(require_at_least=1))
        async with aiohttp.ClientSession() as session:
            async with session.get(self.config["Funkwhale"]["instance_url"] +
                                   f"/api/v1/search?query={search}") as response:
                if response.status >= 400:
                    raise rc.ExternalError(f"Request returned {response.status}")
                j = await response.json()
        if len(j["albums"]) < 1:
            raise rc.UserError("Nessun file audio trovato con il nome richiesto.")
        album = j["albums"][0]
        return [f'{self.config["Funkwhale"]["instance_url"]}{track["listen_url"]}' for track in album["tracks"]]



from .brawlhallametal import BrawlhallaMetal
from .brawlhallatier import BrawlhallaTier


class BrawlhallaRank:
    __slots__ = "metal", "tier"

    def __init__(self, metal: BrawlhallaMetal, tier: BrawlhallaTier):
        self.metal: BrawlhallaMetal = metal
        self.tier: BrawlhallaTier = tier

    def __gt__(self, other):
        if other is None:
            return True
        if not isinstance(other, self.__class__):
            raise TypeError(f"Can't compare {self.__class__.__qualname__} with {other.__class__.__qualname__}")
        if self.metal > other.metal:
            return True
        elif self.metal < other.metal:
            return False
        elif self.tier > other.tier:
            return True
        return False

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, self.__class__):
            raise TypeError(f"Can't compare {self.__class__.__qualname__} with {other.__class__.__qualname__}")
        return self.metal == other.metal and self.tier == other.tier

    def __repr__(self):
        return f"<{self.__class__.__qualname__}: {self.metal} {self.tier}>"

    def __str__(self):
        return f"{self.metal} {self.tier}"



from typing import *
import royalnet.commands as rc


class PmotsCommand(rc.Command):
    name: str = "pmots"

    description: str = "Confondi Proto!"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        await data.reply("👣 pmots pmots")



from typing import *
import royalnet.commands as rc
import aiohttp
import io


class PugCommand(rc.Command):
    name: str = "pug"

    description: str = "Invia un carlino casuale in chat."

    syntax: str = ""

    aliases = ["carlino", "carlo", "mallllco"]

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        async with aiohttp.ClientSession() as session:
            async with session.get("https://dog.ceo/api/breed/pug/images/random") as response:
                if response.status >= 400:
                    raise rc.ExternalError(f"Request returned {response.status}")
                result = await response.json()
                assert "status" in result
                assert result["status"] == "success"
                assert "message" in result
                url = result["message"]
            async with session.get(url) as response:
                img = await response.content.read()
                await data.reply_image(image=io.BytesIO(img))



from typing import *
import royalnet.commands as rc
import royalnet.utils as ru

from ..tables import Diario


class DiarioquoteCommand(rc.Command):
    name: str = "diarioquote"

    description: str = "Cita una riga del diario."

    aliases = ["dq", "quote", "dquote"]

    syntax = "{id}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        try:
            entry_id = int(args[0].lstrip("#"))
        except ValueError:
            raise rc.CommandError("L'id che hai specificato non è valido.")
        entry: Diario = await ru.asyncify(data.session.query(self.alchemy.get(Diario)).get, entry_id)
        if entry is None:
            raise rc.CommandError("Nessuna riga con quell'id trovata.")
        await data.reply(f"ℹ️ {entry}")



import royalnet.utils as ru
from royalnet.backpack.tables import *
from royalnet.constellation.api import *
from ..tables import Cvstats


class ApiCvstatsLatestStar(ApiStar):
    path = "/api/cvstats/latest/v1"

    methods = ["GET"]

    summary = "Get the latest 500 cvstats."

    tags = ["cvstats"]

    async def api(self, data: ApiData) -> ru.JSON:
        CvstatsT = self.alchemy.get(Cvstats)

        cvstats = data.session.query(CvstatsT).order_by(CvstatsT.timestamp.desc()).limit(500).all()

        return list(map(lambda c: c.json(), cvstats))



from typing import *
from sqlalchemy import *
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declared_attr
from ..types import DotaMedal, DotaStars, DotaRank
import steam.steamid


class Dota:
    __tablename__ = "dota"

    @declared_attr
    def _steamid(self):
        return Column(BigInteger, ForeignKey("steam._steamid"), primary_key=True)

    @declared_attr
    def steam(self):
        return relationship("Steam", backref=backref("dota", uselist=False))

    @property
    def steamid(self):
        return steam.steamid.SteamID(self._steamid)

    @declared_attr
    def _rank_tier(self):
        return Column(Integer)

    @property
    def medal(self) -> Optional[DotaMedal]:
        if self._rank_tier is None:
            return None
        return DotaMedal(self._rank_tier // 10)

    @medal.setter
    def medal(self, value: DotaMedal):
        if not isinstance(value, DotaMedal):
            raise AttributeError("medal can only be set to DotaMedal objects.")
        self._rank_tier = value.value * 10 + self.stars.value

    @property
    def stars(self) -> Optional[DotaStars]:
        if self._rank_tier is None:
            return None
        return DotaStars(self._rank_tier % 10)

    @stars.setter
    def stars(self, value: DotaStars):
        if not isinstance(value, DotaStars):
            raise AttributeError("stars can only be set to DotaStars objects.")
        self._rank_tier = self.medal.value * 10 + value.value

    @property
    def rank(self) -> Optional[DotaRank]:
        if self._rank_tier is None:
            return None
        return DotaRank(self.medal, self.stars)

    @rank.setter
    def rank(self, value: Optional[DotaRank]):
        if value is None:
            self._rank_tier = None
            return
        if not isinstance(value, DotaRank):
            raise AttributeError("rank can only be set to DotaRank objects (or None).")
        self._rank_tier = value.rank_tier

    @declared_attr
    def wins(self):
        return Column(Integer)

    @declared_attr
    def losses(self):
        return Column(Integer)

    def json(self):
        rank = self.rank

        return {
            "rank": {
                "raw": self._rank_tier,
                "medal": rank.medal.name,
                "rank": rank.stars.name
            } if self._rank_tier is not None else None,
            "wins": self.wins,
            "losses": self.losses
        }

    def __repr__(self):
        return f"<Dota account {self._steamid}>"

    def __str__(self):
        return f"[c]dota:{self._steamid}[/c]"



from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import declared_attr
import steam.steamid
from ..types import BrawlhallaRank, BrawlhallaTier, BrawlhallaMetal


# noinspection PyAttributeOutsideInit
class Brawlhalla:
    __tablename__ = "brawlhalla"

    @declared_attr
    def brawlhalla_id(self):
        return Column(Integer, primary_key=True)

    @declared_attr
    def _steamid(self):
        return Column(BigInteger, ForeignKey("steam._steamid"), unique=True)

    @declared_attr
    def steam(self):
        return relationship("Steam", backref=backref("brawlhalla", uselist=False))

    @property
    def steamid(self):
        return steam.steamid.SteamID(self._steamid)

    @declared_attr
    def name(self):
        return Column(String, nullable=False)

    @declared_attr
    def rating_1v1(self):
        return Column(Integer)

    @declared_attr
    def tier_1v1(self):
        return Column(Enum(BrawlhallaTier))

    @declared_attr
    def metal_1v1(self):
        return Column(Enum(BrawlhallaMetal))

    @property
    def rank_1v1(self):
        if self.metal_1v1 is None:
            return None
        return BrawlhallaRank(metal=self.metal_1v1, tier=self.tier_1v1)

    @rank_1v1.setter
    def rank_1v1(self, value):
        if not isinstance(value, BrawlhallaRank):
            raise TypeError("rank_1v1 can only be set to BrawlhallaRank values.")
        self.metal_1v1 = value.metal
        self.tier_1v1 = value.tier

    @property
    def duos(self):
        return [*self._duos_one, *self._duos_two]

    @property
    def rating_2v2(self):
        duos = sorted(self.duos, key=lambda d: -d.rating_2v2)
        if len(duos) == 0:
            return None
        return duos[0].rating_2v2

    @property
    def tier_2v2(self):
        duos = sorted(self.duos, key=lambda d: -d.rating_2v2)
        if len(duos) == 0:
            return None
        return duos[0].tier_2v2

    @property
    def metal_2v2(self):
        duos = sorted(self.duos, key=lambda d: -d.rating_2v2)
        if len(duos) == 0:
            return None
        return duos[0].metal_2v2

    @property
    def rank_2v2(self):
        duos = sorted(self.duos, key=lambda d: -d.rating_2v2)
        if len(duos) == 0:
            return None
        return duos[0].rank_2v2

    def json(self):
        one_rank = self.rank_1v1
        two_rank = self.rank_2v2
        return {
            "name": self.name,
            "1v1": {
                "rating": self.rating_1v1,
                "metal": one_rank.metal.name,
                "tier": one_rank.tier.name
            } if one_rank is not None else None,
            "2v2": {
                "rating": self.rating_2v2,
                "metal": two_rank.metal.name,
                "tier": two_rank.tier.name
            } if two_rank is not None else None
        }

    def __repr__(self):
        return f"<Brawlhalla account {self._steamid}>"

    def __str__(self):
        return f"[c]brawlhalla:{self.brawlhalla_id}[/c]"
